@@ -27,9 +27,10 @@ def _xydatobjectarraytostring(arr):
     return dat
 
 class Snake():
-    _gridSize = 50
-    
-    def __init__(self, _pendown=1, homeX = 50 + 25 + 5, homeY = 50 + 25 + 5, canvWidth = 400, canvHeight = 200, \
+
+
+    # If you  are using gridmode=True, the movement will be based on the grid not units (ie forward(1) >>> forward(1*gridSize)) and rotation will rounded to 90 degrees
+    def __init__(self, _pendown=1, gridmode=False, gridsize=50, homeX = 50 + 25 + 5, homeY = 50 + 25 + 5, canvWidth = 400, canvHeight = 200, \
                     turtleMainColor="#00A651", turtleAccentColor="#FFF600", speed = 5, rotspeed = 5, pencolor = 'red', penwidth=3):
         """Initializes the snake. Can be customized with arguments"""
         self._turtleMainColor = turtleMainColor
@@ -40,20 +41,73 @@ class Snake():
         self._pencolor = pencolor
         self._penwidth = penwidth
         self._rotation = 90
+        self._gridsize = gridsize
+        self._gridmode = gridmode
+        
+        if(gridmode and homeX == 80):
+            homeX = 0
+            homeY = 0
+ 
         self._x = homeX
         self._y = homeY
         self._homeX = homeX
         self._homeY = homeY
+        
         self._canvWidth = canvWidth
         self._canvHeight = canvHeight
-        self._gridSize = self._gridSize
         self._actions = []
-        self._levelDataString = []
+        self._levelDataString = [] 
+        
+        self._walls = []
+        self._lava = []
+        
         self._appendCurrentState();
         
+    def drawLevel (self, xSize = 8, ySize = 4, gridSize=50, backgroundColor='white', drawGrid=1,\
+                        apple = [], walls = [], doors = [], lava = [], bridges = [], debug = False):
+        '''
+            Use this to draw a level with a grid + objects
+            for walls and doors and bridges, input should be an array of arrays of size 3 [x, y, data]
+            for lava and apples, input should be an array of arrays of size 2 [x, y]
+            Walls can have 6 orientations as data > lines >> 0 = │; 1 = ─; corners >> 2 = └; 3 = ┌; 4 = ┐; 5 = ┘;
+            doors and bridges can have 2 orientations as data >> 0 = │; 1 = ─;
+            Also you should NOT draw lava under bridge, it comes with it
+            eg walls = [[1,2,0], [1,3,0], [1,4,5]]
+            eg lava = [[1,2], [1,2]]
+        '''
+        self._walls = walls
+        self._lava = lava
+                        
+        self._levelDataString = "["
+        
+        gridBorders = 5 # This is also set like this in the javascript side of the code, so don't change it here
+        self._gridsize = gridSize
+        self._canvWidth = xSize * self._gridsize + gridBorders*2
+        self._canvHeight = ySize * self._gridsize + gridBorders*2
+        
+        self._levelDataString += "[%s, %s, %s, '%s', %s], " % (self._canvWidth, self._canvHeight, self._gridsize, backgroundColor, drawGrid)
+
+        self._levelDataString += _xyobjectarraytostring(apple)
+        self._levelDataString += _xydatobjectarraytostring(walls)
+        self._levelDataString += _xydatobjectarraytostring(doors)
+        self._levelDataString += _xyobjectarraytostring(lava)
+        self._levelDataString += _xydatobjectarraytostring(bridges)
+        
+        self._levelDataString += "]"
+        
+        if(debug):
+            print(self._levelDataString)
+        
     def _appendCurrentState (self):
         """An Internal helper method for 'saving' the current state of the snake"""
-        self._actions.append([self._rotation, self._x, self._y,
+        x = self._x
+        y = self._y
+        
+        if(self._gridmode):
+            x = 5 + round(x)*self._gridsize + self._gridsize/2
+            y = 5 + round(y)*self._gridsize + self._gridsize/2
+        
+        self._actions.append([self._rotation, x, y,
                             self._speed, self._rotspeed,
                             self._pendown, self._pencolor, self._penwidth,
                             self._turtleMainColor, self._turtleAccentColor
@@ -62,12 +116,20 @@ class Snake():
     def goto(self, x, y):
         """An Internal helper method for setting the snake's position."""
         # note that the snake can get outside of the canvas!
-        self._x = round(x, 2)
-        self._y = round(y, 2)
+        if(self._gridmode):
+            self._x = round(x)
+            self._y = round(y)
+        else:
+            self._x = round(x, 2)
+            self._y = round(y, 2)
+            
         self._appendCurrentState()
         
     def setheading(self, rot):
         """An Internal helper method for setting the snake's rotation."""
+        if(self._gridmode):
+            rot = round(rot/90)*90
+        
         self._rotation = round(rot, 2)
         self._appendCurrentState()
         
@@ -134,40 +196,62 @@ class Snake():
         
     def pos(self):
         return self._x, self._y
+    
+    def islocationclear(self,x,y):
+        for wall in self._walls:
+            if(wall[0] == x and wall[1] == y):
+                return False
+                
+        for lava in self._lava:
+            if(lava[0] == x and lava[1] == y):
+                return False
+                
+        return True
+                
         
     
-    def drawLevel (self, xSize = 8, ySize = 4, gridSize=50, backgroundColor='white', drawGrid=1,\
-                        apple = [], walls = [], doors = [], lava = [], bridges = [], debug = False):
-        '''
-            Use this to draw a level with a grid + objects
-            for walls and doors and bridges, input should be an array of arrays of size 3 [x, y, data]
-            for lava and apples, input should be an array of arrays of size 2 [x, y]
-            Walls can have 6 orientations as data > lines >> 0 = │; 1 = ─; corners >> 2 = └; 3 = ┌; 4 = ┐; 5 = ┘;
-            doors and bridges can have 2 orientations as data >> 0 = │; 1 = ─;
-            Also you should NOT draw lava under bridge, it comes with it
-            eg walls = [[1,2,0], [1,3,0], [1,4,5]]
-            eg lava = [[1,2], [1,2]]
-        '''
-                        
-        self._levelDataString = "["
+    def isfrontclear(self):
+        x = self._x
+        y = self._y
+        rot = round((self._rotation%360)/90)
+        if(rot == 0):
+            y+=1
+        elif(rot==1):
+            x+=1
+        elif(rot==2):
+            y-=1
+        else:
+            x-=1
+        return self.islocationclear(x,y)
+    
+    def ifleftclear(self):
+        x = self._x
+        y = self._y
+        rot = round((self._rotation%360)/90)
+        if(rot == 0):
+            x-=1
+        elif(rot==1):
+            y+=1
+        elif(rot==2):
+            x+=1
+        else:
+            y-=1
+        return self.islocationclear(x,y)
+    
+    def isrightclear(self):
+        x = self._x
+        y = self._y
+        rot = round((self._rotation%360)/90)
+        if(rot == 0):
+            x+=1
+        elif(rot==1):
+            y-=1
+        elif(rot==2):
+            x-=1
+        else:
+            y+=1
+        return self.islocationclear(x,y)
         
-        gridBorders = 5 # This is also set like this in the javascript side of the code, so don't change it here
-        self._gridSize = gridSize
-        self._canvWidth = xSize * self._gridSize + gridBorders*2
-        self._canvHeight = ySize * self._gridSize + gridBorders*2
-        
-        self._levelDataString += "[%s, %s, %s, '%s', %s], " % (self._canvWidth, self._canvHeight, self._gridSize, backgroundColor, drawGrid)
-
-        self._levelDataString += _xyobjectarraytostring(apple)
-        self._levelDataString += _xydatobjectarraytostring(walls)
-        self._levelDataString += _xydatobjectarraytostring(doors)
-        self._levelDataString += _xyobjectarraytostring(lava)
-        self._levelDataString += _xydatobjectarraytostring(bridges)
-        
-        self._levelDataString += "]"
-        
-        if(debug):
-            print(self._levelDataString)
     
     def display(self):
         """Displays the snake. This method is required at the end to actually show the snake"""
