@@ -86,16 +86,6 @@ function onFrame(event) {
 			// path to:
 			if (isArrived) {
 				n = n + 1;
-				if (n >= actionData.length){
-					isGoing = false;
-					return;
-				}
-				
-				movDestination = new Point(actionData[n][1], actionData[n][2]);
-				movVector = movDestination - turtle.position;
-				rotDestination = actionData[n][0];
-				
-				processActionData(n);
 				
 				// Death Check
 				if (typeof lavaPoints != 'undefined') {
@@ -112,7 +102,8 @@ function onFrame(event) {
 				// Win Check
 				if (typeof winPlace !== 'undefined') {
 					var distance = (winPlace - turtle.position).length;
-					if(distance < 1){
+					console.log(distance)
+					if(distance < gridSize/2){
 						console.log("You won!");
 						Win();
 						isGoing = false;
@@ -120,6 +111,20 @@ function onFrame(event) {
 					
 				}
 				
+				
+				if (n >= actionData.length){
+					isGoing = false;
+					return;
+				}
+				
+				movDestination = new Point(actionData[n][1], actionData[n][2]);
+				movVector = movDestination - turtle.position;
+				rotDestination = actionData[n][0];
+				
+				processActionData(n);
+				
+				
+				
 				console.log(actionData[n])
 				if(actionData[n-1][0] == actionData[n][0]){
 					movOrRotate = true;
